package Class::SingletonMethod::Std;

use 5.006002; # encourage upgrade
use strict;
use warnings;
use Carp;
use Data::Dumper;

our $VERSION = sprintf "%d.%02d", q$Revision: 2.0 $ =~ /(\d+)/g;
our $DEBUG   = 0;

my %Method;

sub singleton_method {
    my $self = shift;
    my ( $method, $subref ) = @_;
    ref $subref eq 'CODE'
      or croak '$obj->singleton_method(name => sub{...})';
    $Method{ $self + 0 }{$method} = $subref;

}
*add_method = \&singleton_method;

sub remove_method{
    my $self = shift;
    my $method = shift;
    delete  $Method{ $self + 0 }{$method};
    $self;
}

sub can {
    my ( $self, $method ) = @_;
    my $subref = $Method{ $self + 0 }{$method};
    return $subref if $subref;
    return $self->SUPER::can($method);
}

sub DESTROY {
    my $self = shift;
    $DEBUG and carp "Destroing $self";
    delete $Method{ $self + 0 };
}

sub AUTOLOAD {
    my $self   = shift;
    my $method = our $AUTOLOAD;
    $method =~ s/.*:://;
    $DEBUG and carp "($self)->$method";
    if ( my $subref = $Method{ $self + 0 }{$method} ) {
        $subref->( $self, @_ );
    }
    else {
        my $pkg = ref $self;
        croak qq(Can't locate object method "$method" via package "$pkg");
    }
}

1;

__END__

=head1 NAME

Class::SingletonMethod::Std - Singleton methods via base class

=head1 SYNOPSIS

  use base 'Class::SingletonMethod::Std';
  my $a = Some::Class->new; 
  my $b = Some::Class->new; 
     
  $a->singleton_method( dump => sub { 
    my $self = shift; 
    require Data::Dumper; 
    print STDERR Date::Dumper::Dumper($self)  
  }); 
     
  $a->dump; # Prints a representation of the object. 
  $b->dump; # Can't locate method "dump" 

=head1 DESCRIPTION

This module provides a Perl implementation of singleton methods which
is further discussed in L<Class::SingletonMethod>.

Unlike L<Universal::SingletonMethod> which makes the "Singleton Class"
for object, rebless the object to it, and make the original class its
base class,  This module stores the method in inside-out class manner
and dispatch the method via AUTOLOAD.

That way you don't wind up 256 classes for 256 objects with singleton
methods attached.

The catch is that you cannot override the original method that way
because perl calls class methods first.

=head1 AUTHOR

Simon Cozens, C<simon@cpan.org>

Dan Kogai, C<dankogai@dan.co.jp>

=head1 SEE ALSO

ruby L<http://www.ruby-lang.org/>

Advanced Perl Programming, 2nd ed.
L<http://www.oreilly.com/catalog/advperl2/>

=cut
